SystemOrganization addCategory: #'Ma code editor enhancements-Info'!

!Utilities class methodsFor: '*ma code editor enhancements' stamp: 'cmm 2/5/2004 00:46'!
classFromPattern: pattern withCaption: aCaption
	"If there is a class whose name exactly given by pattern, return it.
	If there is only one class in the system whose name matches pattern, return it.
	Otherwise, put up a menu offering the names of all classes that match pattern, and return the class chosen, else nil if nothing chosen.
	This method ignores tab, space, & cr characters in the pattern"

	| toMatch potentialClassNames classNames exactMatch index lines reducedIdentifiers |
	(toMatch _  pattern copyWithoutAll:
			{Character space.  Character cr.  Character tab})
		isEmpty ifTrue: [^ nil].
	Symbol hasInterned: toMatch ifTrue:
		[:patternSymbol | Smalltalk at: patternSymbol ifPresent:
			[:maybeClass | (maybeClass isKindOf: Class) ifTrue: [^ maybeClass]]].

	toMatch _ (toMatch copyWithout: $.) asLowercase.
	potentialClassNames _ Smalltalk classNames asOrderedCollection.
	classNames _ pattern last = $. 
		ifTrue: [potentialClassNames select:
					[:nm |  nm asLowercase = toMatch]]
		ifFalse: [potentialClassNames select: 
					[:n | n includesSubstring: toMatch caseSensitive: false]].
	exactMatch _ classNames detect: [:each | each asLowercase = toMatch] ifNone: [nil].
	lines _ OrderedCollection new.
	exactMatch ifNotNil: [ lines add: 1 ].
	self maMarked: 'system override'.
	(reducedIdentifiers _ pattern maPossibleCodeElementIdentifiers select: [ :each | potentialClassNames includes: each ]) notEmpty
		ifTrue:
			[ classNames addAll: reducedIdentifiers.
			lines add: 1 + classNames size + reducedIdentifiers size ].
	classNames isEmpty ifTrue: [ ^self maClassFromAttemptedEvaluationOf: pattern ].
	index _ classNames size = 1
		ifTrue:	[1]
		ifFalse:	[exactMatch
			ifNil: [(PopUpMenu labelArray: classNames lines: lines) startUpWithCaption: aCaption]
			ifNotNil: [classNames addFirst: exactMatch.
				(PopUpMenu labelArray: classNames lines: lines) startUpWithCaption: aCaption]].
	index = 0 ifTrue: [^ nil].
	^ Smalltalk at: (classNames at: index) asSymbol

"
	Utilities classFromPattern: 'CharRecog'
	Utilities classFromPattern: 'rRecog'
	Utilities classFromPattern: 'znak'
	Utilities classFromPattern: 'orph'
"
! !

!Utilities class methodsFor: '*ma code editor enhancements' stamp: 'cmm 1/14/2002 13:39'!
maClassFromAttemptedEvaluationOf: aString

	| object |
	object _ Compiler evaluate: aString.
	^object isBehavior
		ifTrue:
			[ object ]
		ifFalse:
			[ object
				ifNil: [ nil ]
				ifNotNil: [ object class ] ]
! !

!TextFontChange methodsFor: '*ma code editor enhancements' stamp: 'cmm 1/20/2002 01:20'!
asNextLarger

	^self class fontNumber: self fontNumber + 1
! !

PackageInfo subclass: #MaCodeEditorEnhancementsPackageInfo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma code editor enhancements-Info'!

!MaCodeEditorEnhancementsPackageInfo class methodsFor: 'as yet unclassified' stamp: 'cmm 4/7/2004 23:01'!
initialize

	ParagraphEditor initialize.  "reinitialize key shortcuts to support typeMethodArgument:"! !

!ParagraphEditor class methodsFor: '*ma code editor enhancements' stamp: 'cmm 1/20/2002 01:28'!
initializeCmdKeyShortcuts
	"Initialize the (unshifted) command-key (or alt-key) shortcut table."

	"NOTE: if you don't know what your keyboard generates, use Sensor kbdTest"

	"ParagraphEditor initialize"

	| cmdMap cmds |
	cmdMap := Array new: 256 withAll: #noop:.	"use temp in case of a crash"
	cmdMap at: 1 + 1 put: #cursorHome:.	"home key"
	cmdMap at: 4 + 1 put: #cursorEnd:.	"end key"
	cmdMap at: 8 + 1 put: #backspace:.	"ctrl-H or delete key"
	cmdMap at: 11 + 1 put: #cursorPageUp:.	"page up key"
	cmdMap at: 12 + 1 put: #cursorPageDown:.	"page down key"
	cmdMap at: 13 + 1 put: #crWithIndent:.	"cmd-Return"
	cmdMap at: 27 + 1 put: #offerMenuFromEsc:.	"escape key"
	cmdMap at: 28 + 1 put: #cursorLeft:.	"left arrow key"
	cmdMap at: 29 + 1 put: #cursorRight:.	"right arrow key"
	cmdMap at: 30 + 1 put: #cursorUp:.	"up arrow key"
	cmdMap at: 31 + 1 put: #cursorDown:.	"down arrow key"
	cmdMap at: 32 + 1 put: #selectWord:.	"space bar key"
	cmdMap at: 127 + 1 put: #forwardDelete:.	"del key"
	'1234' do:
		[ :eachChar | cmdMap at: eachChar asciiValue + 1 put: #typeMethodArgument: ].
	'056789-=' 
		do: [:char | cmdMap at: char asciiValue + 1 put: #changeEmphasis:].
	'([{''"<' do: [:char | cmdMap at: char asciiValue + 1 put: #enclose:].
	cmdMap at: $, asciiValue + 1 put: #shiftEnclose:.
	cmds := #($a #selectAll: $b #browseIt: $c #copySelection: $d #doIt: $e #exchange: $f #find: $g #findAgain: $h #setSearchString: $i #inspectIt: $j #doAgainOnce: $k #offerFontMenu: $l #cancel: $m #implementorsOfIt: $n #sendersOfIt: $o #spawnIt: $p #printIt: $q #querySymbol: $r #recognizer: $s #save: $t #tempCommand: $u #align: $v #paste: $w #backWord: $x #cut: $y #swapChars: $z #undo:).
	1 to: cmds size
		by: 2
		do: [:i | cmdMap at: (cmds at: i) asciiValue + 1 put: (cmds at: i + 1)].
	CmdActions := cmdMap! !

!ParagraphEditor methodsFor: '*ma code editor enhancements' stamp: 'cmm 4/7/2004 01:05'!
backWord: characterStream 
	"If the selection is not a caret, delete it and leave it in the backspace buffer.
	 Else if there is typeahead, delete it.
	 Else, delete the word before the caret."

	| indices newPosition |
	self closeTypeIn: characterStream.
	indices _ self 
		setIndices: true
		forward: false.
	newPosition _ 1 max: (indices at: #moving)-1.
	newPosition _  self previousWord: newPosition.
	sensor keyboard.
	self selectMark: (indices at: #fixed) point: newPosition - 1.
	^ true! !

!ParagraphEditor methodsFor: '*ma code editor enhancements' stamp: 'cmm 1/20/2002 17:28'!
methodArgument: anInteger

	^(ReadStream on: self text asString) nextLine substrings 
		at: 2*anInteger
		ifAbsent: [ '' ]
! !

!ParagraphEditor methodsFor: '*ma code editor enhancements' stamp: 'cmm 1/20/2002 17:06'!
typeMethodArgument: characterStream 
	"Replace the current text selection with the name of the method argument represented by the keyCode."

	| keyCode |
	keyCode _ ('1234' 
		indexOf: sensor keyboard 
		ifAbsent: [1]).
	characterStream nextPutAll: (self methodArgument: keyCode).
	^false
! !

!String methodsFor: '*ma code editor enhancements' stamp: 'cm 1/2/2002 18:42'!
maPossibleCodeElementIdentifiers

	| answer |
	answer _ OrderedCollection new.
	self withIndexDo:
		[ :eachChar :index |
		eachChar isUppercase ifTrue: [ answer add: (self
			copyFrom: index
			to: self size) ] ].
	^answer
! !
MaCodeEditorEnhancementsPackageInfo initialize!
